-- Author      : Nogarder
-- Create Date : 2015/3/2 1:29:00
local pairs=pairs
local type=type
local CopyTable=CopyTable
local table_insert=table.insert
local next=next
--
local RS_PageManager_PageList={}
local RS_PageManager_ButtonList={}
local RS_PageManager_LinkList={}
--
local RS_PageManager_TemplateList={}
local RS_PageManager_WindowList={}
--
local RS_PageManager_MinimizeList={}
--
local RS_PageManager_PageFunc={}
--macro for minimize all sub pages
RS.MinimizePages=function(page)
	local closelist={[page]=true}
	while(next(closelist))do
		for opage,_ in pairs(closelist)do
			--insert all pages linked to it
			for apage,parent in pairs(RS_PageManager_LinkList)do
				if(parent==opage)then
					closelist[apage]=true
				end
			end
			--hide it and remove it from the list,(unless switch is true)
			if(opage~=page)then
				RS.Animation.MinimizeScroll(opage.Scroll)
			end
			closelist[opage]=nil
		end
	end
end
--unminimize
RS.RestorePages=function(page)
	local closelist={[page]=true}
	while(next(closelist))do
		for opage,_ in pairs(closelist)do
			--insert all pages linked to it
			for apage,parent in pairs(RS_PageManager_LinkList)do
				if(parent==opage)then
					closelist[apage]=true
				end
			end
			--hide it and remove it from the list,(unless switch is true)
			if(opage~=page)then
				RS.Animation.RestoreScroll(opage.Scroll)
			end
			closelist[opage]=nil
		end
	end
end
--macro for close pages
RS.ClosePages=function(page,nothide)
	local closelist={[page]=true}
	while(next(closelist))do
		for opage,_ in pairs(closelist)do
			--insert all pages linked to it
			for apage,parent in pairs(RS_PageManager_LinkList)do
				if(parent==opage)then
					closelist[apage]=true
				end
			end
			--hide it and remove it from the list,(unless switch is true)
			if((not nothide) or opage~=page)then
				RS.Animation.HideFrame(opage)
			end
			closelist[opage]=nil
		end
	end
end
--
RS.GetPage=function(name)
	return RS_PageManager_PageList[name]
end
--
RS.PageIsShown=function(name)
	return RS_PageManager_PageList[name] and RS_PageManager_PageList[name]:IsShown()
end
--
RS.RegisterPage=function(name,page)
	RS_PageManager_PageList[name]=page
end
--
local RS_AutoMinimize=function(page)
	local apage=RS_PageManager_LinkList[page]
	while(apage)do
		local parent=RS_PageManager_LinkList[apage]
		if(parent and (not RS.Animation.IsMinimized(parent.Scroll)))then
			apage=parent
		else
			if(apage==page)then
				return true --true means no more aviliable parent
			else
				RS.RegisterMinimize(page,apage)
				return apage.Scroll:GetWidth()--return the width
			end
		end
	end
end
RS.SetPage=function(name,parent,arg1,arg2,arg3,arg4,arg5,onhidefunc)
	--check visible
	RS.HideTemplates()
	local page=RS_PageManager_PageList[name]
	--if not inited yet ,do it
	if(not page)then
		RS.RunFunc(RS_PageManager_PageFunc[name])
		page=RS_PageManager_PageList[name]
	end
	if(not page.DontHideOther)then
		RS.ClosePages(parent,true)
	end
	page.LinkTo=parent
	page:Hide()
	page.OnHideFunc=onhidefunc
	RS_PageManager_LinkList[page]=parent
	RS.RunFunc(page.OnSetFunc,arg1,arg2,arg3,arg4,arg5)
	RS.Animation.ShowFrame(page)
	--auto minimize --don't use it for now
	--local uiright=UIParent:GetWidth()
	--local pageright=page.Scroll:GetRight()
	--local result=0
	--while(result~=true and pageright>uiright)do
	--	pageright=pageright-result
	--	result=RS_AutoMinimize(page)
	--end
end
RS.SetPageButton=function(button,name,parent,arg1,arg2,arg3,arg4,arg5)
	if(not RS_PageManager_ButtonList[button])then
		button:HookScript("OnMouseUp",function()
			local data=RS_PageManager_ButtonList[button]
			RS.ClosePages(data.Parent,true)
			RS.SetPage(data.Name,data.Parent,data.Arg1,data.Arg2,data.Arg3,data.Arg4,data.Arg5)
		end)
	end
	RS_PageManager_ButtonList[button]={
		Name=name,
		Parent=parent,
		Arg1=arg1,
		Arg2=arg2,
		Arg3=arg3,
		Arg4=arg4,
		Arg5=arg5
		}		
end
--

--template
RS.RegisterTemplate=function(frame,hidefunc)
	RS_PageManager_TemplateList[frame]=hidefunc
end
--
RS.HideTemplates=function(oframe)
	for frame,func in pairs(RS_PageManager_TemplateList)do
		if(frame~=oframe)then
			func()
		end
	end
end
--
RS.RegisterWindow=function(frame,hidefunc)
	RS_PageManager_WindowList[frame]=hidefunc
end
--
RS.RegisterMinimize=function(frame,minimize)
	if(minimize)then
		RS.Animation.MinimizeScroll(minimize.Scroll)
		if(not RS_PageManager_MinimizeList[minimize])then
			RS_PageManager_MinimizeList[minimize]={}
		end
		RS_PageManager_MinimizeList[minimize][frame]=true
	end
end
RS.UnregisterMinimize=function(frame,minimize)
	if(minimize)then
		if(RS_PageManager_MinimizeList[minimize])then
			RS_PageManager_MinimizeList[minimize][frame]=nil
			if(next(RS_PageManager_MinimizeList[minimize])==nil)then
				RS_PageManager_MinimizeList[minimize]=nil
				RS.Animation.RestoreScroll(minimize.Scroll)
			end
		else
			RS.Animation.RestoreScroll(minimize.Scroll)
		end
	else
		for minimize,data in pairs(RS_PageManager_MinimizeList)do
			data[frame]=nil
			if(next(data)==nil)then
				RS_PageManager_MinimizeList[minimize]=nil
				RS.Animation.RestoreScroll(minimize.Scroll)
			end
		end
	end
end
--
RS.RegisterPageFunc=function(name,func)
	RS_PageManager_PageFunc[name]=func
end